@@ -5,7 +5,7 @@
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -56,7 +55,7 @@ b64enc_start (struct b64state *state, FILE *fp, const char *title)
     {
       state->title = xtrystrdup (title);
       if (!state->title)
-        return  gpg_error_from_errno (errno);
+        return  gpg_error_from_syserror ();
     }
   return 0;
 }
@@ -129,7 +128,7 @@ b64enc_write (struct b64state *state, const void *buffer, size_t nbytes)
   return 0;
 
  write_error:
-  return gpg_error_from_errno (errno);
+  return gpg_error_from_syserror ();
 }
 
 gpg_error_t
@@ -199,7 +198,7 @@ b64enc_finish (struct b64state *state)
   goto cleanup;
 
  write_error:
-  err = gpg_error_from_errno (errno);
+  err = gpg_error_from_syserror ();
 
  cleanup:
   if (state->title)
