@@ -1,12 +1,12 @@
 /* keyserver.c - generic keyserver code
- * Copyright (C) 2001, 2002, 2003, 2004, 2005,
- *               2006 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005, 2006,
+ *               2007 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
  * GnuPG is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * GnuPG is distributed in the hope that it will be useful,
@@ -15,9 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
- * USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -45,6 +43,13 @@
 #include "dns-cert.h"
 #include "pka.h"
 
+#ifdef HAVE_W32_SYSTEM
+/* It seems Vista doesn't grok X_OK and so fails access() tests.
+   Previous versions interpreted X_OK as F_OK anyway, so we'll just
+   use F_OK directly. */
+#undef X_OK
+#define X_OK F_OK
+#endif /* HAVE_W32_SYSTEM */
 
 struct keyrec
 {
@@ -678,9 +683,9 @@ parse_keyrec(char *keystring)
       while(*tok)
 	if(tok[0]=='%' && tok[1] && tok[2])
 	  {
-	    if((userid[i]=hextobyte(&tok[1]))==-1)
-	      userid[i]='?';
+            int c;
 
+	    userid[i] = (c=hextobyte(&tok[1])) == -1 ? '?' : c;
 	    i++;
 	    tok+=3;
 	  }
@@ -963,7 +968,7 @@ keyserver_spawn(enum ks_action action,strlist_t list,KEYDB_SEARCH_DESC *desc,
   byte *line=NULL;
   struct exec_info *spawn;
   const char *scheme;
-  const char *libexecdir = get_libexecdir ();
+  const char *libexecdir = gnupg_libexecdir ();
 
   assert(keyserver);
 
@@ -989,7 +994,7 @@ keyserver_spawn(enum ks_action action,strlist_t list,KEYDB_SEARCH_DESC *desc,
      After some more thinking about this we came to the conclusion
      that it is better to load the helpers from the directory where
      the program of this process lives.  Fortunately Windows provides
-     a way to retrieve this and our get_libexecdir function has been
+     a way to retrieve this and our gnupg_libexecdir function has been
      modified to return just this.  Setting the exec-path is not
      anymore required.  
        set_exec_path(libexecdir);
