@@ -1,6 +1,6 @@
 /* gpgsm.c - GnuPG for S/MIME 
  * Copyright (C) 2001, 2002, 2003, 2004, 2005,
- *               2006, 2007  Free Software Foundation, Inc.
+ *               2006, 2007, 2008  Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -108,6 +108,7 @@ enum cmd_and_opt_values {
   oFixedPassphrase,
   oLogFile,
   oNoLogFile,
+  oAuditLog,
 
   oEnableSpecialFilenames,
 
@@ -117,9 +118,11 @@ enum cmd_and_opt_values {
   oTTYtype,
   oLCctype,
   oLCmessages,
+  oXauthority,
 
   oPreferSystemDirmngr,
   oDirmngrProgram,
+  oDisableDirmngr,
   oProtectToolProgram,
   oFakedSystemTime,
 
@@ -147,7 +150,6 @@ enum cmd_and_opt_values {
   oEnablePolicyChecks,
   oAutoIssuerKeyRetrieve,
   
-
   oTextmode,
   oFingerprint,
   oWithFingerprint,
@@ -172,6 +174,7 @@ enum cmd_and_opt_values {
   oOpenPGP,
   oCipherAlgo,
   oDigestAlgo,
+  oExtraDigestAlgo,
   oCompressAlgo,
   oCommandFD,
   oNoVerbose,
@@ -228,6 +231,7 @@ enum cmd_and_opt_values {
   oIgnoreTimeConflict,
   oNoRandomSeedFile,
   oNoAutoKeyRetrieve,
+  oNoCommonCertsImport,
   oUseAgent,
   oMergeOnly,
   oTryAllSecrets,
@@ -336,12 +340,13 @@ static ARGPARSE_OPTS opts[] = {
     { oTextmode, "textmode",  0, N_("use canonical text mode")},
 #endif
 
-    { oOutput, "output",    2, N_("use as output file")},
+    { oOutput, "output",    2, N_("|FILE|write output to FILE")},
     { oVerbose, "verbose",   0, N_("verbose") },
     { oQuiet,	"quiet",   0, N_("be somewhat more quiet") },
     { oNoTTY, "no-tty", 0, N_("don't use the terminal at all") },
-    { oLogFile, "log-file"   ,2, N_("use a log file for the server")},
+    { oLogFile, "log-file"   ,2, N_("|FILE|write a server mode log to FILE")},
     { oNoLogFile, "no-log-file" ,0, "@"},
+    { oAuditLog, "audit-log", 2, N_("|FILE|write an audit log to FILE")},
 #if 0
     { oForceV3Sigs, "force-v3-sigs", 0, N_("force v3 signatures") },
     { oForceMDC, "force-mdc", 0, N_("always use a MDC for encryption") },
@@ -385,6 +390,7 @@ static ARGPARSE_OPTS opts[] = {
     { oCipherAlgo, "cipher-algo", 2 , N_("|NAME|use cipher algorithm NAME")},
     { oDigestAlgo, "digest-algo", 2 ,
       N_("|NAME|use message digest algorithm NAME")},
+    { oExtraDigestAlgo, "extra-digest-algo", 2 , "@" },
 #if 0
     { oCompressAlgo, "compress-algo", 1 , N_("|N|use compress algorithm N")},
 #endif
@@ -424,11 +430,12 @@ static ARGPARSE_OPTS opts[] = {
     { oTTYtype,    "ttytype",     2, "@" },
     { oLCctype,    "lc-ctype",    2, "@" },
     { oLCmessages, "lc-messages", 2, "@" },
+    { oXauthority, "xauthority", 2, "@" },
     { oDirmngrProgram, "dirmngr-program", 2 , "@" },
+    { oDisableDirmngr, "disable-dirmngr", 0 , "@" },
     { oProtectToolProgram, "protect-tool-program", 2 , "@" },
     { oFakedSystemTime, "faked-system-time", 2, "@" }, /* (epoch time) */
 
-
     { oNoBatch, "no-batch", 0, "@" },
     { oWithColons, "with-colons", 0, "@"},
     { oWithKeyData,"with-key-data", 0, "@"},
@@ -456,6 +463,7 @@ static ARGPARSE_OPTS opts[] = {
     { oListOnly, "list-only", 0, "@"},
     { oIgnoreTimeConflict, "ignore-time-conflict", 0, "@" },
     { oNoRandomSeedFile,  "no-random-seed-file", 0, "@" },
+    { oNoCommonCertsImport, "no-common-certs-import", 0, "@" },
 {0} };
 
 
@@ -481,6 +489,7 @@ static int default_validation_model;
 
 static char *build_list (const char *text,
 			 const char *(*mapf)(int), int (*chkf)(int));
+static char *build_lib_list (const char *text);
 static void set_cmd (enum cmd_and_opt_values *ret_cmd,
                      enum cmd_and_opt_values new_cmd );
 
@@ -553,7 +562,7 @@ our_md_test_algo (int algo)
 static const char *
 my_strusage( int level )
 {
-  static char *digests, *pubkeys, *ciphers;
+  static char *digests, *pubkeys, *ciphers, *libs;
   const char *p;
 
   switch (level)
@@ -593,7 +602,12 @@ my_strusage( int level )
         digests = build_list("Hash: ", gcry_md_algo_name, our_md_test_algo );
       p = digests;
       break;
-      
+    case 38:
+      if (!libs)
+        libs = build_lib_list(_("Used libraries:"));
+      p = libs;
+      break;
+     
     default: p = NULL; break;
     }
   return p;
@@ -632,6 +646,49 @@ build_list (const char *text, const char * (*mapf)(int), int (*chkf)(int))
   return list;
 }
 
+static char *
+build_lib_list (const char *text)
+{
+  struct { const char *name; const char *version; } array[5];
+  int idx;
+  size_t n;
+  char *list, *p;
+
+  if (maybe_setuid)
+    gcry_control (GCRYCTL_INIT_SECMEM, 0, 0);  /* Drop setuid. */
+
+  idx = 0;
+  array[idx].name = "gcrypt";
+  array[idx++].version = gcry_check_version (NULL);
+  array[idx].name = "ksba";
+  array[idx++].version = ksba_check_version (NULL);
+  array[idx].name = "assuan";
+  array[idx++].version = GNUPG_LIBASSUAN_VERSION;
+  array[idx].name = NULL;
+  array[idx++].version = NULL;
+
+  n = strlen (text) + 1;
+  for (idx=0; array[idx].name; idx++)
+    {
+      n += 2 + strlen (array[idx].name);
+      if (array[idx].version)
+        n += 1 + strlen (array[idx].version) + 1;
+    }
+  n++;
+  list = xmalloc (n+1);
+  p = stpcpy (stpcpy (list, text), " ");
+  for (idx=0; array[idx].name; idx++)
+    {
+      if (idx)
+        p = stpcpy (p, ", ");
+      p = stpcpy (p, array[idx].name);
+      if (array[idx].version)
+        p = stpcpy (stpcpy (stpcpy (p, "("), array[idx].version), ")");
+    }
+  strcpy (p, "\n");
+  return list;
+}
+
 
 /* Set the file pointer into binary mode if required.  */
 static void
@@ -782,21 +839,26 @@ main ( int argc, char **argv)
   int default_config =1;
   int default_keyring = 1;
   char *logfile = NULL;
+  char *auditlog = NULL;
   int greeting = 0;
   int nogreeting = 0;
   int debug_wait = 0;
   int use_random_seed = 1;
+  int no_common_certs_import = 0;
   int with_fpr = 0;
   char *def_digest_string = NULL;
+  char *extra_digest_algo = NULL;
   enum cmd_and_opt_values cmd = 0;
   struct server_control_s ctrl;
   certlist_t recplist = NULL;
   certlist_t signerlist = NULL;
   int do_not_setup_keys = 0;
   int recp_required = 0;
+  estream_t auditfp = NULL;
 
   /*mtrace();*/
 
+  gnupg_reopen_std ("gpgsm");
   /* trap_unaligned ();*/
   gnupg_rl_initialize ();
   set_strusage (my_strusage);
@@ -1102,6 +1164,8 @@ main ( int argc, char **argv)
         case oLogFile: logfile = pargs.r.ret_str; break;
         case oNoLogFile: logfile = NULL; break;          
 
+        case oAuditLog: auditlog = pargs.r.ret_str; break;
+
         case oBatch: 
           opt.batch = 1;
           greeting = 0;
@@ -1152,7 +1216,9 @@ main ( int argc, char **argv)
         case oTTYtype: opt.ttytype = xstrdup (pargs.r.ret_str); break;
         case oLCctype: opt.lc_ctype = xstrdup (pargs.r.ret_str); break;
         case oLCmessages: opt.lc_messages = xstrdup (pargs.r.ret_str); break;
+        case oXauthority: opt.xauthority = xstrdup (pargs.r.ret_str); break;
         case oDirmngrProgram: opt.dirmngr_program = pargs.r.ret_str;  break;
+        case oDisableDirmngr: opt.disable_dirmngr = 1;  break;
         case oPreferSystemDirmngr: opt.prefer_system_dirmngr = 1; break;
         case oProtectToolProgram:
           opt.protect_tool_program = pargs.r.ret_str; 
@@ -1239,8 +1305,13 @@ main ( int argc, char **argv)
           }
           break;
 
+        case oExtraDigestAlgo: 
+          extra_digest_algo = pargs.r.ret_str;
+          break;
+
         case oIgnoreTimeConflict: opt.ignore_time_conflict = 1; break;
         case oNoRandomSeedFile: use_random_seed = 0; break;
+        case oNoCommonCertsImport: no_common_certs_import = 1; break;
 
         case oEnableSpecialFilenames: allow_special_filenames =1; break;
 
@@ -1294,6 +1365,11 @@ main ( int argc, char **argv)
     }
 #  endif
 
+  if (auditlog)
+    log_info ("NOTE: The audit log feature (--audit-log) is "
+              "WORK IN PRORESS and not ready for use!\n");
+
+
   if (may_coredump && !opt.quiet)
     log_info (_("WARNING: program may create a core file!\n"));
 
@@ -1377,6 +1453,12 @@ main ( int argc, char **argv)
           if (our_md_test_algo(opt.def_digest_algo) )
             log_error (_("selected digest algorithm is invalid\n"));
         }
+      if (extra_digest_algo)
+        {
+          opt.extra_digest_algo = gcry_md_map_name (extra_digest_algo);
+          if (our_md_test_algo (opt.extra_digest_algo) )
+            log_error (_("selected digest algorithm is invalid\n"));
+        }
     }
 
   if (log_get_errorcount(0))
@@ -1399,7 +1481,7 @@ main ( int argc, char **argv)
       int created;
 
       keydb_add_resource ("pubring.kbx", 0, 0, &created);
-      if (created)
+      if (created && !no_common_certs_import)
         {
           /* Import the standard certificates for a new default keybox. */
           char *filelist[2];
@@ -1419,6 +1501,26 @@ main ( int argc, char **argv)
     keydb_add_resource (sl->d, 0, 0, NULL);
   FREE_STRLIST(nrings);
 
+
+  /* Prepare the audit log feature for certain commands.  */
+  if (auditlog)
+    {
+      switch (cmd)
+        {
+        case aEncr: 
+        case aSign:
+        case aDecrypt:
+        case aVerify:
+          audit_release (ctrl.audit);
+          ctrl.audit = audit_new ();
+          auditfp = open_es_fwrite (auditlog);
+          break;
+        default:
+          break;
+        }
+    }
+
+
   if (!do_not_setup_keys)
     {
       for (sl = locusr; sl ; sl = sl->next)
@@ -1465,6 +1567,7 @@ main ( int argc, char **argv)
   
   fname = argc? *argv : NULL;
   
+  /* Dispatch command.  */
   switch (cmd)
     {
     case aGPGConfList: 
@@ -1495,6 +1598,8 @@ main ( int argc, char **argv)
                 GC_OPT_FLAG_NONE );
         printf ("auto-issuer-key-retrieve:%lu:\n",
                 GC_OPT_FLAG_NONE );
+        printf ("disable-dirmngr:%lu:\n",
+                GC_OPT_FLAG_NONE );
 #ifndef HAVE_W32_SYSTEM
         printf ("prefer-system-dirmngr:%lu:\n",
                 GC_OPT_FLAG_NONE );
@@ -1767,6 +1872,15 @@ main ( int argc, char **argv)
         log_error ("invalid command (there is no implicit command)\n");
 	break;
     }
+
+  /* Print the audit result if needed.  */
+  if (auditlog && auditfp)
+    {
+      audit_print_result (ctrl.audit, auditfp, 0);
+      audit_release (ctrl.audit);
+      ctrl.audit = NULL;
+      es_fclose (auditfp);
+    }
   
   /* cleanup */
   gpgsm_release_certlist (recplist);
